@@ -1,16 +1,21 @@
 """Command line interface."""
 
+import contextlib
 import threading
 from pathlib import Path
-from typing import List, Optional
+from typing import Optional
 
-import steam.steamid
 import xdg
 from libcli import BaseCLI
 from loguru import logger
 
+import tf2mon
+import tf2mon.controls
 import tf2mon.layouts
 from tf2mon.conlog import Conlog
+from tf2mon.controls import Controls
+from tf2mon.database import Database
+from tf2mon.hacker import HackerManager
 from tf2mon.logger import configure_logger
 from tf2mon.monitor import Monitor
 from tf2mon.steamweb import SteamWebAPI
@@ -37,15 +42,51 @@ class CLI(BaseCLI):
         "con_logfile": Path("console.log"),
         #
         # databases.
-        "players": _cachedir / "steamplayers.db",
-        "hackers-base": _cachedir / "hackers-base.json",
-        "hackers-local": _cachedir / "hackers-local.json",
+        "database": _cachedir / "tf2mon.db",
+        "hackers": _cachedir / "hackers.json",
         "exclude-file": Path(__file__).parent / "data" / "exclude.txt",
         "webapi_key": "",
         # this player.
         "player_name": "Bad Dad",
     }
 
+    # Create all controls.
+    controls = Controls("tf2mon.controls", suffix="Control")
+
+    # Bind some controls.
+    controls.bind("HelpControl", "F1")
+    controls.bind("MotdControl", "Ctrl+F1")
+    controls.bind("DebugFlagControl", "F2")
+    controls.bind("TauntFlagControl", "F3")
+    controls.bind("ThroeFlagControl", "Shift+F3")
+    controls.bind("ShowKDControl", "F4")
+    controls.bind("ShowKillsControl", "Shift+F4")
+    controls.bind("UserPanelControl", "F5")
+    controls.bind("JoinOtherTeamControl", "F6")
+    controls.bind("SortOrderControl", "F7")
+    controls.bind("LogLocationControl", "F8")
+    controls.bind("LogLevelControl", "Shift+F8")
+    controls.bind("ResetPaddingControl", "Ctrl+F8")
+    controls.bind("GridLayoutControl", "F9")
+    controls.bind("ShowDebugControl", "KP_INS")
+    controls.bind("SingleStepControl", "KP_DEL")
+    controls.bind("KickLastCheaterControl", "[", game_only=True)
+    controls.bind("KickLastRacistControl", "]", game_only=True)
+    controls.bind("KickLastSuspectControl", "\\", game_only=True)
+    controls.bind("KicksPopControl", "KP_HOME")
+    controls.bind("KicksClearControl", "KP_LEFTARROW")
+    controls.bind("KicksPopleftControl", "KP_END")
+    controls.bind("SpamsPopControl", "KP_PGUP")
+    controls.bind("SpamsClearControl", "KP_RIGHTARROW")
+    controls.bind("SpamsPopleftControl", "KP_PGDN")
+    # controls.bind("PullControl", "KP_UPARROW")
+    controls.bind("ClearQueuesControl", "KP_5")
+    controls.bind("PushControl", "KP_DOWNARROW")
+
+    # def debug(self, text: str) -> None:
+    #     """Override to silence."""
+    # for --merge-hackers
+
     def init_logging(self, verbose: int) -> None:
         """Set logging levels based on `--verbose`."""
 
@@ -61,25 +102,26 @@ class CLI(BaseCLI):
             prog=__package__,
             description=self.dedent(
                 """
-    Team Fortress II (`TF2`) Console Monitor (`%(prog)s`) is an interactive
-    terminal application that displays real-time game state and player
-    statistics of an actively running TF2 game.
+    Team Fortress II (`TF2`) Console Monitor, `%(prog)s`, is an interactive
+    terminal application that displays scoreboards and player statistics of
+    an active game. `%(prog)s` can also `--rewind` and `--single-step`
+    through previous games.
 
-    `%(prog)s` recognizes and tracks `cathook-bots`, and provides both
-    application and in-game key-bindings to quickly `CALLVOTE KICK`
-    hackers, mark players as racist, suspect, etc.
+    `%(prog)s` recognizes and tracks abusive players, such as racists and
+    `cathook-bots`. It provides application and in-game key-bindings to
+    `CALLVOTE KICK` cheaters/hackers, or mark as racist or suspect.
 
     `%(prog)s` reads TF2's console logfile (`con_logfile`), creates action
-    scripts in TF2's `cfg` directory, and binds keys for the gamer to press
-    in-game to take those actions; such as issue `SAY` and `CALLVOTE KICK`
+    scripts in TF2's `cfg/user` directory, and binds keys for gamer to
+    press to take those actions, such as issue `SAY` and `CALLVOTE KICK`
     commands.
 
     Other in-game key-bindings taunt gamer's last victim/killer with a
-    `CHAT` message customized with their name, k/d ratio, weapon and duel-
-    score (`Taunt` and `Throe`).
+    `CHAT` message customized with their name, k/d ratio, rotating spam,
+    weapon and duel-scores (`Taunt` and `Throe`).
 
-    By default, `%(prog)s` starts reading the con_logfile from its end
-    (`--no-rewind`), and continues to `--follow` it until interrupted.
+    By default, `%(prog)s` starts reading `con_logfile` from its end
+    (`--no-rewind`), and `--follow`s its tail.
                 """
             ),
         )
@@ -132,13 +174,7 @@ class CLI(BaseCLI):
         )
         self.add_default_to_help(arg)
 
-        arg = self.parser.add_argument(
-            "--layout",
-            choices=[x.name for x in list(tf2mon.layouts.LAYOUT_ENUM)],
-            default=list(tf2mon.layouts.LAYOUT_ENUM)[0].name,
-            help="choose display layout",
-        )
-        self.add_default_to_help(arg)
+        self.controls.add_arguments_to(self.parser)
 
         arg = self.parser.add_argument(
             "con_logfile",
@@ -222,38 +258,43 @@ class CLI(BaseCLI):
         group = self.parser.add_argument_group("Database options")
 
         arg = group.add_argument(
-            "--players",
+            "--database",
             metavar="FILE",
-            default=Path(self.config["players"]),
+            default=Path(self.config["database"]),
             type=Path,
-            help="cache `steam` user data",
+            help="main database",
         )
         self.add_default_to_help(arg)
 
         arg = group.add_argument(
-            "--hackers-base",
+            "--hackers",
             metavar="FILE",
-            default=Path(self.config["hackers-base"]),
+            default=Path(self.config["hackers"]),
             type=Path,
-            help="upstream hackers database",
-        )
-        self.add_default_to_help(arg)
-
-        arg = group.add_argument(
-            "--hackers-local",
-            metavar="FILE",
-            default=Path(self.config["hackers-local"]),
-            type=Path,
-            help="local hackers database",
+            help="hackers database",
         )
         self.add_default_to_help(arg)
 
         group.add_argument(
-            "--print-steamid",
+            "--print-steamids",
+            nargs="+",
             metavar="STEAMID",
             help="print `ISteamUser.GetPlayerSummaries` for `STEAMID` and exit",
         )
 
+        group.add_argument(
+            "--print-hackers",
+            action="store_true",
+            help="print hackers database and exit",
+        )
+
+        # group.add_argument(
+        #     "--merge-hackers",
+        #     metavar="FILE",
+        #     type=Path,
+        #     help="merge `FILE` with `--hackers FILE` to `stdout` and exit",
+        # )
+
         self.parser.add_argument_group(
             "Configuration file",
             self.dedent(
@@ -281,24 +322,23 @@ class CLI(BaseCLI):
         Queues -->    Kicks     Admin     Spams
                         |         |         |
                         v         v         v
-                   +-----------------------------+
+                   +---------+---------+---------+
         last/      |         |         |         |
-        newest --> |   pop   |  pull   |   pop   |
+        newest --> |   pop   |         |   pop   |
                    |         |         |         |
-                   |---------+---------+---------|
+                   +---------+---------+---------+
                    |         |         |         |
                    |  clear  |  clear  |  clear  |
                    |         |  both   |         |
-                   |---------+---------+---------|
+                   +---------+---------+---------+
         first/     |         |         |         |
-        oldest --> | popleft |  push   | popleft |
+        oldest --> | popleft |  vet    | popleft |
                    |         |         |         |
-                   +-----------------------------+
+                   +---------+---------+---------+
 
-    To vet new players that have joined the game, and to remove inactive
-    players, press `NUMPAD-DOWNARROW`. The monitor indicates when new
-    players have arrived and that key should be pressed; or press it each
-    time you die.
+    To vet new players that have joined the game, `%(prog)s` needs their
+    `steamid`s, and to get them, gamer must press `NUMPAD-DOWNARROW`. The
+    `status` panel will be highlighted when the monitor needs steamids.
 
     When detected, `%(prog)s` pushes hackers onto the `Kicks` queue, and
     alerts the gamer, who may then press `HOME`/`END` to issue `CHAT`
@@ -344,25 +384,8 @@ class CLI(BaseCLI):
 
         self.parser.add_argument_group(
             "Function Keys",
-            self.dedent(
-                """
-    These function keys are available in-game and in the monitor:
-
-        F1 Display help.
-        F2 Toggle Debug (control `say` vs `echo`).
-        F3 Enable/disable Taunts and Throes.
-        F4 Include kd-ratio in messages.
-        F5 Control User-panel display: Kicks, Spams, Duels and Auto.
-        F6 Join other team.
-        F7 Change scoreboard sort column.
-        F8 Change logger location format.
-        F9 Change grid layout.
-        [  Kick last killer as cheater.
-        ]  Kick last killer as racist.
-        \\  Mark last killer as suspect.
-        KP_DEL Begin single-stepping.
-                """
-            ),
+            "These function keys are available in-game and in the monitor:\n\n"
+            + self.controls.fkey_help(),
         )
 
         self.parser.add_argument_group(
@@ -432,9 +455,6 @@ class CLI(BaseCLI):
     def main(self) -> None:
         """Command line interface entry point (method)."""
 
-        # if "webapi_key" not in self.config:
-        #     self.parser.error("Missing config `webapi_key`")
-
         if self.options.con_logfile == self.config["con_logfile"]:
             # Not given on command line; prefix with effective parent.
             # (configured default is a basename relative to `tf2_install_dir`)
@@ -460,27 +480,31 @@ class CLI(BaseCLI):
             logger.info(f"con_logfile {str(self.options.con_logfile)!r} cleaned; Exiting.")
             self.parser.exit()
 
-        if self.options.print_steamid:
-            api = SteamWebAPI(
-                dbpath=self.options.players,
-                webapi_key=self.config.get("webapi_key"),
-            )
-            api.connect()
-            steamid = steam.steamid.SteamID(self.options.print_steamid)
-            if not steamid.is_valid():
-                self.parser.error(f"invalid steamid `{self.options.print_steamid}`")
-            steam_player = api.find_steamid(steamid)
-            print(steam_player)
+        if self.options.print_steamids:
+            Database(self.options.database)
+            api = SteamWebAPI(webapi_key=self.config.get("webapi_key"))
+            for steamid in self.options.print_steamids:
+                print(api.fetch_steamid(steamid))
             self.parser.exit()
 
-        if self.options.layout:  # get enum from name
-            self.options.layout = tf2mon.layouts.LAYOUT_ENUM.__dict__[self.options.layout]
+        if self.options.print_hackers:
+            Database(self.options.database)
+            hackers = HackerManager(self.options.hackers)
+            with contextlib.suppress(BrokenPipeError):
+                hackers.print_report()
+            self.parser.exit()
 
-        monitor = Monitor(self)
-        monitor.run()
+        # if self.options.merge_hackers:
+        #     hackers = HackerManager(self.options.hackers)
+        #     hackers.load(self.options.merge_hackers)
+        #     print(str(hackers))
+        #     self.parser.exit()
+
+        tf2mon.monitor = Monitor(self)
+        tf2mon.monitor.run()
 
 
-def main(args: Optional[List[str]] = None) -> None:
+def main(args: Optional[list[str]] = None) -> None:
     """Command line interface entry point (function)."""
 
     threading.current_thread().name = "MAIN"
