@@ -2,11 +2,10 @@
 
 from loguru import logger
 
+import tf2mon
 from tf2mon.chat import Chat
-from tf2mon.hacker import HackerAttr
+from tf2mon.player import Player
 from tf2mon.regex import Regex
-from tf2mon.steamplayer import steamid_from_str
-from tf2mon.user import Team, UserState
 
 
 class Gameplay:
@@ -14,35 +13,24 @@ class Gameplay:
 
     # pylint: disable=line-too-long
 
-    def __init__(self, monitor):
+    def __init__(self):
         """Initialize Gameplay."""
 
         leader = (
             r"^(\d{2}/\d{2}/\d{4} - \d{2}:\d{2}:\d{2}: )?"  # anchor to head; optional timestamp
         )
 
-        self.monitor = monitor
-
         self.regex_list = [
-            # key_listboundkeys
-            # Regex(
-            #    '^".*" = ".*"$',
-            #    lambda m: logger.debug('key_listboundkeys')),
             # new server
             Regex(
                 leader + "(Client reached server_spawn.$|Connected to [0-9])",
-                lambda m: self.monitor.reset_game(),
+                lambda m: tf2mon.monitor.reset_game(),
             ),
             # capture/defend
             Regex(
                 leader
-                + r"(?P<username>.*) (?P<action>(?:captured|defended)) (?P<capture_pt>.*) for team #(?P<teamno>\d)$",  # noqa
-                lambda m: self.capture(
-                    m.group("username"),
-                    m.group("action"),
-                    m.group("capture_pt"),
-                    int(m.group("teamno")),
-                ),
+                + r"(?P<username>.*) (?P<action>(?:captured|defended)) (?P<capture_pt>.*) for team #(?P<s_teamno>\d)$",  # noqa
+                lambda m: self.capture(*m.groups()),
             ),
             # must be before `chat`
             # account : not logged in  (No account specified)
@@ -55,74 +43,61 @@ class Gameplay:
             # edicts  : 1378 used of 2048 max
             Regex(
                 leader + r"(account|version|map|udp\/ip|tags|steamid|players|edicts)\s+: (.*)",
-                lambda m: logger.log("server", m.group(0)),
+                lambda m: ...,  # logger.log("server", m.group(0)),
             ),
             # "06/05/2022 - 13:54:19: Client ping times:"
             Regex(
                 leader + r"Client ping times:",
-                lambda m: logger.log("server", m.group(0)),
+                lambda m: ...,  # logger.log("server", m.group(0)),
             ),
             # "06/05/2022 - 13:54:19:   67 ms : luft"
             # "06/05/2022 - 13:54:19:xy 87 ms : BananaHatTaco"
             Regex(
                 leader + r"\s*\d+ ms .*",
-                lambda m: logger.log("server", m.group(0)),
+                lambda m: ...,  # logger.log("server", m.group(0)),
             ),
             # chat
-            # 'Bad Dad :  hello'
-            # '*DEAD* Bad Dad :  hello'
-            # '*DEAD*(TEAM) Bad Dad :  hello'
+            # 'Bob :  hello'
+            # '*DEAD* Bob :  hello'
+            # '*DEAD*(TEAM) Bob :  hello'
             Regex(
                 leader
                 + r"(?:(?P<dead>\*DEAD\*)?(?P<teamflag>\(TEAM\))? )?(?P<username>.*) :  ?(?P<msg>.*)$",  # noqa
-                lambda m: self.playerchat(
-                    m.group("teamflag"), m.group("username"), m.group("msg")
-                ),
+                lambda m: self.playerchat(*m.groups()),
             ),
             # kill
             Regex(
                 leader
                 + r"(?P<killer>.*) killed (?P<victim>.*) with (?P<weapon>.*)\.(?P<crit> \(crit\))?$",  # noqa
-                lambda m: self.kill(
-                    m.group("killer"), m.group("victim"), m.group("weapon"), m.group("crit")
-                ),
+                lambda m: self.kill(*m.groups()),
             ),
             # connected
             Regex(
                 leader + "(?P<username>.*) connected$",
-                lambda m: self.connected(m.group("username")),
+                lambda m: self.connected(*m.groups()),
             ),
             # status
             # "# userid name                uniqueid            connected ping loss state"
-            # "#     29 "Bad Dad"           [U:1:42708103]      01:24       67    0 active"
+            # "#     29 "Bob"               [U:1:99999999]      01:24       67    0 active"
+            # "#    158 "Jones"             [U:1:9999999999]     2:21:27    78    0 active
             Regex(
                 leader
-                + r'#\s*(?P<userid>\d+) "(?P<username>.+)"\s+(?P<steamid>\S+)\s+(?P<elapsed>[\d:]+)\s+(?P<ping>\d+)',  # noqa
-                lambda m: self.status(
-                    int(m.group("userid")),
-                    m.group("username"),
-                    m.group("steamid"),
-                    m.group("ping"),
-                ),
+                + r'#\s*(?P<s_userid>\d+) "(?P<username>.+)"\s+(?P<steamid>\S+)\s+(?P<elapsed>[\d:]+)\s+(?P<ping>\d+)',  # noqa
+                lambda m: self.status(*m.groups()),
             ),
             # status
             # "# userid name                uniqueid            connected ping loss state"
             # "#      3 "Nobody"            BOT                                     active
             Regex(
                 leader + r'#\s*(?P<userid>\d+) "(?P<username>.+)"\s+(?P<steamid>BOT)\s+active',
-                lambda m: self.status(
-                    int(m.group("userid")),
-                    m.group("username"),
-                    m.group("steamid"),
-                    0,
-                ),
+                lambda m: self.status(*m.groups(), "", 0),
             ),
             # tf_lobby_debug
-            # "Member[22] [U:1:42708103]  team = TF_GC_TEAM_INVADERS  type = MATCH_PLAYER"
+            # "Member[22] [U:1:99999999]  team = TF_GC_TEAM_INVADERS  type = MATCH_PLAYER"
             Regex(
                 leader
-                + r"\s*(Member|Pending)\[\d+\] (?P<steamid>\S+)\s+team = (?P<teamname>\w+)",
-                lambda m: self.lobby(m.group("steamid"), m.group("teamname")),
+                + r"\s*(?:Member|Pending)\[\d+\] (?P<steamid>\S+)\s+team = (?P<teamname>\w+)",
+                lambda m: self.lobby(*m.groups()),
             ),
             Regex(
                 leader + "Failed to find lobby shared object",
@@ -131,46 +106,43 @@ class Gameplay:
             #
             # Regex(
             #    '^Teams have been switched',
-            #    lambda m: self.monitor.switch_teams()),
+            #    lambda m: tf2mon.monitor.switch_teams()),
             #
             Regex(
                 leader + r"You have switched to team (?P<teamname>\w+) and will",
-                lambda m: self.monitor.me.assign_team(m.group("teamname")),
+                lambda m: tf2mon.monitor.me.assign_team(m.group("teamname")),
             ),
             # hostname: Valve Matchmaking Server (Virginia iad-1/srcds148 #53)
             Regex(
                 "^hostname: (.*)",
-                lambda m: (
-                    logger.log("server", m.group(0)),
-                    self.monitor.users.check_status(),
-                ),
+                lambda m: tf2mon.monitor.users.check_status(),
             ),
             # "FFD700[RTD] FF4040your mother rolled 32CD32PowerPlay."
             Regex(
                 r"[0-9A-F]{6}\[RTD\] [0-9A-F]{6}(?P<username>.*) rolled [0-9A-F]{6}(?P<perk>.*)",  # noqa
-                lambda m: self.perk(m.group("username"), m.group("perk")),
+                lambda m: self.perk(*m.groups()),
             ),
             Regex(
                 r"[0-9A-F]{6}\[RTD\] [0-9A-F]{6}(?P<username>.*)\'s perk has worn off.",
-                lambda m: self.perk(m.group("username"), None),
+                lambda m: self.perk(*m.groups(), None),
             ),
             Regex(
                 r"[0-9A-F]{6}\[RTD\] [0-9A-F]{6}(?P<username>.*) has changed class during their roll.",  # noqa
-                lambda m: self.perk(m.group("username"), None),
+                lambda m: self.perk(*m.groups(), None),
             ),
             Regex(
                 r"[0-9A-F]{6}\[RTD\] Your perk has worn off.", lambda m: self.perk(None, None)
             ),
         ]
 
-    def capture(self, username, action, capture_pt, teamno):
+    def capture(self, _leader, username, action, capture_pt, s_teamno):
         """Handle message."""
 
         for name in username.split(", "):  # fix: names containing commas
 
-            user = self.monitor.users.find_username(name)
+            user = tf2mon.monitor.users.find_username(name)
 
-            user.assign_teamno(teamno)
+            user.assign_teamno(int(s_teamno))
 
             if action == "captured":
                 user.ncaptures += 1
@@ -178,25 +150,27 @@ class Gameplay:
             else:
                 user.ndefenses += 1
                 level = "DEF"
+            user.dirty = True
             level += user.team.name
 
             logger.log(level, f"{user} {capture_pt!r}")
             user.actions.append(f"{level} {capture_pt!r}")
 
-    def playerchat(self, teamflag, username, msg):
+    def playerchat(self, _leader, _dead, teamflag, username, msg):
         """Handle message."""
 
-        user = self.monitor.users.find_username(username)
+        user = tf2mon.monitor.users.find_username(username)
         chat = Chat(user, teamflag, msg)
 
         user.chats.append(chat)
-        self.monitor.ui.show_chat(chat)
+        tf2mon.monitor.chats.append(chat)
+        tf2mon.monitor.ui.show_chat(chat)
 
         # if this is a team chat, then we know we're on the same team, and
         # if one of us knows which team we're on and the other doesn't, we
         # can assign.
 
-        me = my = self.monitor.me
+        me = my = tf2mon.monitor.me
         if chat.teamflag and user != me:
             # we're on the same team
             if not user.team:
@@ -206,30 +180,20 @@ class Gameplay:
                 me.assign_team(user.team)
 
         # inspect msg
-        if self.monitor.is_racist_text(chat.msg):
-            user.kick(HackerAttr.RACIST)
-
-        # If this looks like a milenko chat, mark him to be tracked when
-        # his steamid becomes available. Doing this now to notify the
-        # operator asap to `TF2MON-PUSH` steamids to us.
-        #
-        # Why track them? No requirement; curious to see relationship
-        # between names and steamids... one-to-one, many-to-one?
-
-        elif user.is_milenko_chat(chat):
-            user.kick(HackerAttr.MILENKO)
+        if tf2mon.monitor.is_racist_text(chat.msg):
+            user.kick(Player.RACIST)
 
         elif user.is_cheater_chat(chat):
-            user.kick(HackerAttr.CHEATER)
+            user.kick(Player.CHEATER)
 
-    def kill(self, s_killer: str, s_victim: str, weapon: str, s_crit: str) -> None:
+    def kill(self, _leader, s_killer: str, s_victim: str, weapon: str, s_crit: str) -> None:
         """Handle message."""
 
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-statements
 
-        killer = self.monitor.users.find_username(s_killer)
-        victim = self.monitor.users.find_username(s_victim)
+        killer = tf2mon.monitor.users.find_username(s_killer)
+        victim = tf2mon.monitor.users.find_username(s_victim)
 
         killer.last_victim = victim
         victim.last_killer = killer
@@ -247,6 +211,8 @@ class Gameplay:
 
         killer.nkills += 1
         victim.ndeaths += 1
+        killer.dirty = True
+        victim.dirty = True
 
         _k = killer.nkills
         _d = killer.ndeaths
@@ -276,14 +242,14 @@ class Gameplay:
 
         # subtotal opponents by weapon_state -----------------------------------
 
-        if role := self.monitor.role_by_weapon.get(weapon):
+        if role := tf2mon.monitor.role_by_weapon.get(weapon):
             killer.role = role
-            if killer.role == self.monitor.sniper_role:
+            if killer.role == tf2mon.monitor.sniper_role:
                 killer.nsnipes += 1
         else:
             role = killer.role
             if weapon not in ("player", "world"):
-                logger.error(f"cannot map {weapon} for {killer}")
+                logger.error(f"cannot map {weapon} for {killer} {role}")
 
         crit = bool(s_crit)
         weapon_state = role.get_weapon_state(weapon, crit, killer.perk)
@@ -308,56 +274,49 @@ class Gameplay:
             weapon_state,
         )
 
-        if killer == self.monitor.me:
-            self.monitor.spammer.taunt(victim, weapon, crit)
+        if tf2mon.monitor.controls["ShowKillsControl"].value:
+            level = "KILL"
+            if killer.team:
+                level += killer.team.name
+            tf2mon.monitor.ui.show_journal(
+                level,
+                f"         {killer.moniker!r:25} killed {victim.moniker!r:25} {weapon_state!r}",
+            )
 
-        if victim == self.monitor.me:
-            self.monitor.spammer.throe(killer, weapon, crit)
+        if killer == tf2mon.monitor.me:
+            tf2mon.monitor.spammer.taunt(victim, weapon, crit)
+
+        if victim == tf2mon.monitor.me:
+            tf2mon.monitor.spammer.throe(killer, weapon, crit)
 
         if not victim.team and killer.team:
             victim.assign_team(killer.opposing_team)
         elif not killer.team and victim.team:
             killer.assign_team(victim.opposing_team)
 
-    def connected(self, username):
+    def connected(self, _leader, username):
         """Handle message."""
 
-        logger.log("CONNECT", self.monitor.users.find_username(username))
+        logger.log("CONNECT", tf2mon.monitor.users.find_username(username))
 
-        self.monitor.ui.notify_operator = True
+        tf2mon.monitor.ui.notify_operator = True
 
-    def status(self, userid, username, s_steamid, ping):
+    def status(self, _leader, userid, username, steamid, elapsed, ping):
         """Handle message."""
 
-        self.monitor.ui.notify_operator = False
-        if not (steamid := steamid_from_str(s_steamid)):
-            return  # invalid
+        # pylint: disable=too-many-arguments
 
-        self.monitor.users.status(userid, username, steamid, ping)
+        tf2mon.monitor.users.status(userid, username, steamid, elapsed, ping)
 
-    def lobby(self, s_steamid, teamname):
+    def lobby(self, _leader, steamid, teamname):
         """Handle message."""
 
-        # this will not be called for games on local server with bots
-        # or community servers; only on valve matchmaking servers.
-
-        if not (steamid := steamid_from_str(s_steamid)):
-            return  # invalid
-
-        if teamname == "TF_GC_TEAM_INVADERS":
-            team = Team.BLU
-        elif teamname == "TF_GC_TEAM_DEFENDERS":
-            team = Team.RED
-        else:
-            logger.critical(f"bad teamname {teamname!r} steamid {steamid}")
-            return
-
-        self.monitor.users.lobby(steamid, team)
+        tf2mon.monitor.users.lobby(steamid, teamname)
 
     def perk(self, username, perk):
         """Handle message."""
 
-        user = self.monitor.users.find_username(username) if username else self.monitor.me
+        user = tf2mon.monitor.users.find_username(username) if username else tf2mon.monitor.me
 
         if perk:
             logger.log("PERK-ON", f"{user} {perk!r}")
@@ -365,44 +324,4 @@ class Gameplay:
             logger.log("PERK-OFF", f"{user} {user.perk!r}")
 
         user.perk = perk
-
-    def repl(self):
-        """Read the console log file and play game."""
-
-        self.monitor.steam_web_api.connect()
-        self.monitor.conlog.open()
-
-        #
-        while (line := self.monitor.conlog.readline()) is not None:
-
-            if not line:  # blank line
-                continue
-
-            regex = Regex.search_list(line, self.monitor.regex_list)
-            if not regex:
-                logger.log("ignore", self.monitor.conlog.last_line)
-                continue
-
-            self.monitor.admin.step(line)
-
-            regex.handler(regex.re_match_obj)
-
-            # Vet all unvetted users that can be vetted, and perform all
-            # postponed work that can be performed.
-
-            for user in self.monitor.users.active_users():
-
-                if not user.vetted and user.steamid:
-                    user.vet_player()
-
-                if user.vetted and user.work_attr:
-                    user.kick()
-
-                if user.state == UserState.DELETE:
-                    self.monitor.users.delete(user)
-
-            # push work to the game
-            self.monitor.msgqueues.send()
-
-            #
-            self.monitor.ui.update_display()
+        user.dirty = True
