@@ -2,71 +2,23 @@
 
 from loguru import logger
 
-from tf2mon.steamplayer import SteamPlayer
-from tf2mon.ui import SORT_ORDER
-from tf2mon.user import User, UserState
+import tf2mon
+from tf2mon.steamid import BOT_STEAMID, parse_steamid
+from tf2mon.user import Team, User, UserState
 
 
 class UserManager:
     """Collection of `User` objects."""
 
-    # pylint: disable=too-many-instance-attributes
-
-    def __init__(self, monitor):
+    def __init__(self):
         """Initialize `User` manager."""
 
-        self.monitor = monitor
-
-        # The pool.
-
         self._users_by_username = {}
         self._users_by_userid = {}
         self._users_by_steamid = {}
         self._teams_by_steamid = {}
         self.kicked_userids = {}
 
-        # Some methods return a sorted list of users.
-        # Names must match scoreboard column headings.
-
-        self._sort_keys = {
-            SORT_ORDER.K: lambda x: (-x.nkills, x.username_upper),
-            SORT_ORDER.KD: lambda x: (-x.kdratio, -x.nkills, x.username_upper),
-            SORT_ORDER.STEAMID: lambda x: (x.steamid.id if x.steamid else 0, x.username_upper),
-            SORT_ORDER.USERNAME: lambda x: x.username_upper,
-        }
-        self._sort_key = self._sort_keys[SORT_ORDER.KD]
-
-    def set_sort_order(self, sort_order):
-        """Set sort-key used by `active_team_users`."""
-
-        self._sort_key = self._sort_keys.get(sort_order)
-
-    def delete(self, user):
-        """Remove `user` from pool."""
-
-        try:
-            if user.last_killer and user.last_killer.last_victim == user:
-                user.last_killer.last_victim = None
-
-            if user.last_victim and user.last_victim.last_killer == user:
-                user.last_victim.last_killer = None
-
-            if user.cloner:
-                user.cloner.clonee = None
-
-            if user.clonee:
-                user.clonee.cloner = None
-
-            del self._users_by_username[user.username]
-            del self._users_by_userid[user.userid]
-
-            if user.steamid:
-                del self._users_by_steamid[user.steamid]
-                del self._teams_by_steamid[user.steamid]
-
-        except KeyError:
-            pass
-
     def find_username(self, username):
         """Return `User` with the matching `username` from pool.
 
@@ -77,33 +29,45 @@ class UserManager:
         username = username.replace(";", ".")
 
         if not (user := self._users_by_username.get(username)):
-            user = User(self.monitor, username)
+            user = User(username)
             self._users_by_username[user.username] = user
             logger.log("ADDUSER", user)
 
         # reset inactivity counter
+        if user.state == UserState.INACTIVE:
+            logger.debug(f"Active again {user}")
+            user.state = UserState.ACTIVE
         user.n_status_checks = 0
         return user
 
-    def status(self, userid, username, steamid, ping) -> None:
+    def status(self, s_userid, username, s_steamid, s_elapsed: str, ping) -> None:
         """Respond to `gameplay.status` event."""
 
+        # pylint: disable=too-many-arguments
+        # pylint: disable=too-many-branches
+        # pylint: disable=too-many-locals
+
+        tf2mon.monitor.ui.notify_operator = False
+
+        if not (steamid := parse_steamid(s_steamid)):
+            return  # invalid
+
+        userid = int(s_userid)
         user = None
 
-        if steamid.id != SteamPlayer.BOT_STEAMID and (
-            user := self._users_by_steamid.get(steamid)
-        ):
+        if steamid != BOT_STEAMID and (user := self._users_by_steamid.get(steamid)):
             if user.username and user.username != username:
-                logger.warning(f"{steamid} changed username from {user.username} to {username}")
+                logger.warning(f"{steamid.id} change username `{user.username}` to `{username}`")
+                user.username = username
+                if user.player:
+                    user.player.track_appearance(username)
+
             if user.userid and user.userid != userid:
-                logger.warning(f"{steamid} changed userid from {user.userid} to {userid}")
+                logger.warning(f"{steamid.id} change userid `{user.userid}` to `{userid}`")
+                user.userid = userid
 
         if not user:
             user = self.find_username(username)
-            if user.userid and user.userid != userid:
-                logger.warning(f"{username} changed userid from {user.userid} to {userid}")
-            if user.steamid and user.steamid != steamid:
-                logger.error(f"{username} changed steamid from {user.steamid} to {steamid}")
 
         if not user.userid:
             user.userid = userid
@@ -113,6 +77,29 @@ class UserManager:
             user.steamid = steamid
             self._users_by_steamid[steamid] = user
 
+        #
+        mdy = s_elapsed.split(":")
+        if len(mdy) == 2:
+            _h, _m, _s = 0, int(mdy[0]), int(mdy[1])
+            user.elapsed = (_m * 60) + _s
+        elif len(mdy) == 3:
+            _h, _m, _s = int(mdy[0]), int(mdy[1]), int(mdy[2])
+            user.elapsed = (_h * 3600) + (_m * 60) + _s
+        else:
+            _h, _m, _s = 0, 0, 0
+            user.elapsed = 0
+
+        # hh:mm:ss
+        #     0:00
+        _ss = f"{_s:02}"
+        if not _h:
+            _mm = f"{_m:5}"
+            user.s_elapsed = _mm + ":" + _ss
+        else:
+            _hh = f"{_h:2}"
+            _mm = f"{_m:02}"
+            user.s_elapsed = _hh + ":" + _mm + ":" + _ss
+
         #
         user.ping = ping
         logger.log("STATUS", user)
@@ -121,16 +108,34 @@ class UserManager:
         if not user.team and (team := self._teams_by_steamid.get(steamid)):
             user.assign_team(team)
 
-    def lobby(self, steamid, team):
+        #
+        if not user.steamplayer:
+            user.vet()
+
+    def lobby(self, s_steamid, teamname):
         """Respond to `gameplay.lobby` event."""
 
+        # this will not be called for games on local server with bots
+        # or community servers; only on valve matchmaking servers.
+
+        if not (steamid := parse_steamid(s_steamid)):
+            return  # invalid
+
+        if teamname == "TF_GC_TEAM_INVADERS":
+            team = Team.BLU
+        elif teamname == "TF_GC_TEAM_DEFENDERS":
+            team = Team.RED
+        else:
+            logger.critical(f"bad teamname {teamname!r} steamid {steamid}")
+            return
+
         if old_team := self._teams_by_steamid.get(steamid):
             # if we've seen this steamid before...
             if old_team != team:
                 # ...and
-                logger.warning(f"{steamid} changed team from {old_team} to {team}")
+                logger.warning(f"{steamid.id} change team `{old_team}` to `{team}`")
         else:
-            logger.log("ADDLOBBY", f"{team} {steamid}")
+            logger.log("ADDLOBBY", f"{team} {steamid.id}")
 
         #
         self._teams_by_steamid[steamid] = team
@@ -143,15 +148,13 @@ class UserManager:
     def active_team_users(self, team):
         """Return list of active users on `team`."""
 
-        assert self._sort_key
-
         yield from sorted(
             [
                 x
                 for x in self._users_by_username.values()
                 if x.state == UserState.ACTIVE and x.team == team
             ],
-            key=self._sort_key,
+            key=tf2mon.monitor.controls["SortOrderControl"].value,
         )
 
     def kick_userid(self, userid, attr):
@@ -174,15 +177,12 @@ class UserManager:
         """
 
         for user in list(self._users_by_username.values()):
-            if user == self.monitor.me:
+            if user == tf2mon.monitor.me:
                 continue
             user.n_status_checks += 1
-            if user.n_status_checks >= self._max_status_checks:
-                logger.log(
-                    "Inactive",
-                    f"{user.n_status_checks} >= {self._max_status_checks} {user.username!r}",
-                )
-                self.delete(user)
+            if user.n_status_checks == self._max_status_checks:
+                logger.log("INACTIVE", user)
+                user.state = UserState.INACTIVE
 
     def switch_teams(self):
         """Switch teams."""
